@@ -2,9 +2,12 @@ import math
 
 noop = lambda i: i
 
-def paginate(items, per_page=10, process=noop, extra={}, query_dict={}):
+def paginate(items, per_page=25, process=noop, extra={}, query_dict={}):
     page = int(query_dict.get('page') or 1)
-    per_page = int(query_dict.get('per_page') or 0) or per_page
+    if 'per_page' in query_dict:
+        per_page = int(query_dict['per_page'])
+    if per_page == 0:
+        per_page = int(1e16)
     offset = (page - 1) * per_page
     total = len(items)
     return {
