@@ -52,7 +52,10 @@ def schema_form(request, form_class, object_id=None, method=None, content_type=N
         model = _meta.model
     kwargs = {}
     if object_id:
-        kwargs['instance'] = model.objects.get(id=object_id)
+        if hasattr(form_class, 'get_one'):
+            kwargs['instance'] = form_class.get_one(object_id)
+        else:
+            kwargs['instance'] = model.objects.get(id=object_id)
     if getattr(_meta, 'login_required', None) and not request.user.is_authenticated:
         print("DEPRECATION WARNING: user form.user_can_METHOD = 'AUTH' instead of meta option.")
         return JsonResponse({'error': 'You must be logged in to do this'}, status=403)
@@ -77,6 +80,16 @@ def schema_form(request, form_class, object_id=None, method=None, content_type=N
             return True
         return f and f(instance, request.user)
 
+    def process(instance):
+        out = { 'id': instance.id }
+        form = form_class(instance=instance)
+        for field_name in form.Meta.fields:
+            out[field_name] = get_default_value(form, field_name)
+        for field_name in getattr(form, 'readonly_fields', []):
+            # TODO should this be on the model or the form?
+            out[field_name] = getattr(instance, field_name)
+        return out
+
     if request.method == "POST" or request.method == "PUT":
         # POST/PUT /api/schema/MODEL/ or /api/schema/MODEL/PK/
         if kwargs.get('instance') and not check_permission('PUT'):
@@ -86,16 +99,18 @@ def schema_form(request, form_class, object_id=None, method=None, content_type=N
         if content_type == 'application/json':
             data = json.loads(request.body.decode('utf-8') or "{}")
             form = form_class(data, **kwargs)
+            form.request_data = data
         else:
             form = form_class(request.POST, request.FILES, **kwargs)
+            form.request_data = request.POST
 
         form.request = request
         if form.is_valid():
             instance = form.save()
-            data = {}
-            if instance:
-                data = {'id': instance.id, 'name': str(instance)}
-            return JsonResponse(data)
+            if not instance:
+                # not all forms are model forms
+                return JsonResponse({})
+            return JsonResponse(process(instance))
         errors = { k: v[0] for k, v in form.errors.get_json_data().items()}
         return JsonResponse({'errors': errors}, status=400)
 
@@ -114,16 +129,6 @@ def schema_form(request, form_class, object_id=None, method=None, content_type=N
         schema = form_to_schema(form_class(**kwargs))
         return JsonResponse({'schema': schema})
 
-    def process(instance):
-        out = { 'id': instance.id }
-        form = form_class(instance=instance)
-        for field_name in form.Meta.fields:
-            out[field_name] = get_default_value(form, field_name)
-        for field_name in getattr(form, 'readonly_fields', []):
-            # TODO should this be on the model or the form?
-            out[field_name] = getattr(instance, field_name)
-        return out
-
     if kwargs.get('instance'):
         # /api/schema/MODEL/PK/
         return JsonResponse(process(form_class(**kwargs).instance))
